@@ -1,4 +1,4 @@
-//! Implementation of `cargo run-ci` subcommand.
+//! Implementation of `cargo-run-ci`.
 
 use anyhow::bail;
 use cargo_util::ProcessBuilder;
@@ -12,7 +12,7 @@ use crate::{util, CIResult};
 pub fn exec(opts: RunOpts) -> CIResult<()> {
     let deps_path = util::target_path(&opts.target, &opts.release)?;
     let binary_paths = util::scan_path(&deps_path, |path| {
-        path.executable() && path.is_file() && util::file_stem_unwrapped(path).contains("-ci")
+        path.executable() && path.is_file() && util::file_stem(path).contains("-ci")
     })?;
     if binary_paths.is_empty() {
         bail!(CIError::BinaryNotFound);
@@ -20,7 +20,7 @@ pub fn exec(opts: RunOpts) -> CIResult<()> {
 
     let binary_names = binary_paths
         .iter()
-        .map(util::file_stem_unwrapped)
+        .map(util::file_stem)
         .map(|mut path| {
             remove_ci(&mut path);
             path
@@ -30,16 +30,20 @@ pub fn exec(opts: RunOpts) -> CIResult<()> {
 
     if let Some(binary_name) = opts.bin {
         for path in binary_paths {
-            let mut name = util::file_name_unwrapped(&path);
+            let mut name = util::file_name(&path);
             remove_ci(&mut name);
             if binary_name == name {
-                return ProcessBuilder::new(&path).exec_replace();
+                return ProcessBuilder::new(&path)
+                    .args(&opts.args.unwrap_or_default())
+                    .exec_replace();
             }
         }
 
         bail!(CIError::BinaryNotAvailable(binary_name, binary_names));
     } else if binary_paths.len() == 1 {
-        return ProcessBuilder::new(&binary_paths[0]).exec_replace();
+        return ProcessBuilder::new(&binary_paths[0])
+            .args(&opts.args.unwrap_or_default())
+            .exec_replace();
     }
 
     bail!(CIError::BinaryNotDetermine(binary_names));
